/******************************************************************************
;  *       @型号                 : MC51F7424
;  *       @创建日期             : 2022.12.22
;  *       @公司/作者            : SINOMCU.COM-AE
;  *       @晟矽微官网           : http://www.sinomcu.com/
;  *       @版权                 : 晟矽微电 版权所有.
;  *----------------------摘要描述----------------------------------
;  *TIMER0初始化,并实现两个软定时(10ms,100ms)                                                  
******************************************************************************/                      

#include "User_Def.h"

#define  DELAY_10MS_NUM      10                  // unit ms
#define  DELAY_100MS_NUM     100                 // unit ms

SoftTimerStatus DELAY_10MS_FLAG = SF_TIMER_STOP; // defalut soft timer stop
SoftTimerStatus DELAY_100MS_FLAG = SF_TIMER_STOP;

static __IO uint8_t delay_10ms_count=0;
static __IO uint8_t delay_100ms_count=0;
uint16_t timer0_init_value=0;

/**
  * @brief System Clock Selection
  * @param clk_div,can be one of following values:
  *         @arg @ref FHS_DIV_1
  *         @arg @ref FHS_DIV_2
  *         @arg @ref FHS_DIV_4
  *         @arg @ref FHS_DIV_8 
  *         @arg @ref FHS_DIV_16
  *         @arg @ref FHS_DIV_32
  *         @arg @ref FHS_DIV_64
  *         @arg @ref FHS_DIV_128 
  * @retval
  * None
  */
#if( FSYS_CLOCK !=6000000UL)
void Set_FHS_ClkDiv(uint8_t clk_div) {
    OSCSS &= ~FHS_DIV_MASK;
    OSCSS |= clk_div;
}
#endif

/**
  * @brief :Get timer frequency division selection
  * @param
  * @retval :uint8_t,can be one of following values:
  *         @arg @ref 1
  *         @arg @ref 2
  *         @arg @ref 8 
  *         @arg @ref 16
  */
uint8_t  Get_Time0_ClkDiv(void) {
  uint8_t xdata return_value = 1;
  switch(TCON1 & TIME0_CLK_DIV_MASK) {
    case TIME0_CLK_DIV_1:
      return_value= 1;
      break;
    case TIME0_CLK_DIV_2: 
      return_value= 2; 
      break;
    case TIME0_CLK_DIV_8: 
      return_value= 8; 
      break;
    case TIME0_CLK_DIV_16: 
      return_value= 16; 
      break;
    default:
      break; 
  }
  return return_value;
} 

/**
  * @brief set timer frequency division
  * @param uint8_t,can be one of following values:
  *         @arg @ref TIME0_CLK_DIV_1
  *         @arg @ref TIME0_CLK_DIV_2
  *         @arg @ref TIME0_CLK_DIV_8 
  *         @arg @ref TIME0_CLK_DIV_16
  * @retval :None
  */
void  Set_Time0_ClkDiv(uint8_t clk_div) {
    TCON1 &= ~TIME0_CLK_DIV_MASK;
    TCON1 |= clk_div; 
} 

/**
  * @brief Time 0 init
  * @param uint16_t,timer_Hz,uint Hz
  * @retval 
  */
void TIM0_Init(uint16_t timer_Hz) {
  uint32_t xdata timer_init_temp=0;

  Time0_16Bit();
  Set_Time0_ClkDiv(TIME0_CLK_DIV_8);

  timer_init_temp = (FSYS_CLOCK / Get_Time0_ClkDiv() ) /timer_Hz;
  timer0_init_value = 65535-timer_init_temp;
  Timer0_Stop();
  TL0 = (uint8_t)timer0_init_value;
  TH0 = (uint8_t)(timer0_init_value>>8);
  Timer0_Start();
  Timer0_IE_En();
}

 /**
  * @brief Timer0 Interruption function.
  * @param
  * @retval
  */
void Timer0_ISR(void) interrupt T0_IRQn {
  Timer0_Stop();
  TL0 = (uint8_t)timer0_init_value;
  TH0 = (uint8_t)(timer0_init_value>>8);
  Timer0_Start();

  if(DELAY_10MS_FLAG != SF_TIMER_STOP) {
    delay_10ms_count++;
    if(delay_10ms_count == DELAY_10MS_NUM ) {    // soft timer OV
      delay_10ms_count =0;
      DELAY_10MS_FLAG =SF_TIMER_OV;
    }
  }

  if(DELAY_100MS_FLAG != SF_TIMER_STOP) {
    delay_100ms_count++;
    if(delay_100ms_count == DELAY_100MS_NUM ) {  // soft timer OV
      delay_100ms_count =0;
      DELAY_100MS_FLAG =SF_TIMER_OV;
    }
  }
}

/**
  * @brief  Start soft timer 10ms,clean ov
  * @param
  * @retval
  */
void Start_10MS_SF_Timer(void) {
  DELAY_10MS_FLAG = SF_TIMER_RUN;
}

/**
  * @brief  Stop soft timer 10ms
  * @param
  * @retval
  */
void Stop_10MS_SF_Timer(void) {
  DELAY_10MS_FLAG = SF_TIMER_STOP;
}

/**
  * @brief  Get state timer 10ms
  * @param 
  * @retval Status of timer 10ms
  */
SoftTimerStatus Get_State_10MS_SF_Timer(void) {
  return DELAY_10MS_FLAG;
}

/**
  * @brief  Start soft timer 100ms,clean ov
  * @param
  * @retval
  */
void Start_100MS_SF_Timer(void) {
  DELAY_100MS_FLAG = SF_TIMER_RUN;
}

/**
  * @brief  Stop soft timer 100ms
  * @param
  * @retval
  */
void Stop_100MS_SF_Timer(void) {
  DELAY_100MS_FLAG = SF_TIMER_STOP;
}

/**
  * @brief  Get state timer 100ms
  * @param
  * @retval Status of timer 100ms
  */
SoftTimerStatus Get_State_100MS_SF_Timer(void) {
  return DELAY_100MS_FLAG;
}

/**************************** end of file *********************************************/
